@@ -11,7 +11,7 @@ module VagrantPlugins
   module ProviderLibvirt
     class Config < Vagrant.plugin('2', :config)
       # manually specify URI
-      # will supercede most other options if provided
+      # will supersede most other options if provided
       attr_accessor :uri
 
       # A hypervisor name to access via Libvirt.
@@ -99,6 +99,7 @@ module VagrantPlugins
       attr_accessor :machine_virtual_size
       attr_accessor :disk_bus
       attr_accessor :disk_device
+      attr_accessor :disk_controller_model
       attr_accessor :disk_driver_opts
       attr_accessor :nic_model_type
       attr_accessor :nested
@@ -128,6 +129,9 @@ module VagrantPlugins
       attr_accessor :tpm_path
       attr_accessor :tpm_version
 
+      # Configure sysinfo values
+      attr_accessor :sysinfo
+
       # Configure the memballoon
       attr_accessor :memballoon_enabled
       attr_accessor :memballoon_model
@@ -195,6 +199,9 @@ module VagrantPlugins
       # serial consoles
       attr_accessor :serials
 
+      # internal helper attributes
+      attr_accessor :host_device_exclude_prefixes
+
       def initialize
         @uri               = UNSET_VALUE
         @driver            = UNSET_VALUE
@@ -253,6 +260,7 @@ module VagrantPlugins
         @machine_virtual_size = UNSET_VALUE
         @disk_bus          = UNSET_VALUE
         @disk_device       = UNSET_VALUE
+        @disk_controller_model = UNSET_VALUE
         @disk_driver_opts  = {}
         @nic_model_type    = UNSET_VALUE
         @nested            = UNSET_VALUE
@@ -280,6 +288,8 @@ module VagrantPlugins
         @tpm_path          = UNSET_VALUE
         @tpm_version       = UNSET_VALUE
 
+        @sysinfo           = UNSET_VALUE
+
         @memballoon_enabled = UNSET_VALUE
         @memballoon_model   = UNSET_VALUE
         @memballoon_pci_bus = UNSET_VALUE
@@ -342,6 +352,9 @@ module VagrantPlugins
         @qemu_use_agent  = UNSET_VALUE
 
         @serials           = UNSET_VALUE
+
+        # internal options to help override behaviour
+        @host_device_exclude_prefixes = UNSET_VALUE
       end
 
       def boot(device)
@@ -769,7 +782,7 @@ module VagrantPlugins
           finalize_id_ssh_key_file
 
           uri += '+ssh://'
-          uri += "#{@username}@" if @username && @username != UNSET_VALUE
+          uri += "#{URI.encode_www_form_component(@username)}@" if @username && @username != UNSET_VALUE
 
           uri += (@host && @host != UNSET_VALUE ? @host : 'localhost')
 
@@ -865,8 +878,15 @@ module VagrantPlugins
         @machine_type = nil if @machine_type == UNSET_VALUE
         @machine_arch = nil if @machine_arch == UNSET_VALUE
         @machine_virtual_size = nil if @machine_virtual_size == UNSET_VALUE
-        @disk_bus = 'virtio' if @disk_bus == UNSET_VALUE
-        @disk_device = 'vda' if @disk_device == UNSET_VALUE
+        @disk_device = @disk_bus == 'scsi' ? 'sda' : 'vda' if @disk_device == UNSET_VALUE
+        @disk_bus = @disk_device.start_with?('sd') ? 'scsi' : 'virtio' if @disk_bus == UNSET_VALUE
+        if @disk_controller_model == UNSET_VALUE
+          if @disk_bus == 'scsi' or @disk_device.start_with?('sd') == 'sd'
+            @disk_controller_model = 'virtio-scsi'
+          else
+            @disk_controller_model = nil
+          end
+        end
         @disk_driver_opts = {} if @disk_driver_opts == UNSET_VALUE
         @nic_model_type = nil if @nic_model_type == UNSET_VALUE
         @nested = false if @nested == UNSET_VALUE
@@ -876,8 +896,7 @@ module VagrantPlugins
         @initrd = '' if @initrd == UNSET_VALUE
         @dtb = nil if @dtb == UNSET_VALUE
         @graphics_type = 'vnc' if @graphics_type == UNSET_VALUE
-        @graphics_autoport = 'yes' if @graphics_port == UNSET_VALUE
-        @graphics_autoport = 'no' if @graphics_port != UNSET_VALUE
+        @graphics_autoport = @graphics_port == UNSET_VALUE ? 'yes' : nil
         if (@graphics_type != 'vnc' && @graphics_type != 'spice') ||
            @graphics_passwd == UNSET_VALUE
           @graphics_passwd = nil
@@ -902,6 +921,8 @@ module VagrantPlugins
         @nic_adapter_count = 8 if @nic_adapter_count == UNSET_VALUE
         @emulator_path = nil if @emulator_path == UNSET_VALUE
 
+        @sysinfo = {} if @sysinfo == UNSET_VALUE
+
         # Boot order
         @boot_order = [] if @boot_order == UNSET_VALUE
 
@@ -972,6 +993,8 @@ module VagrantPlugins
         @qemu_use_agent = false if @qemu_use_agent == UNSET_VALUE
 
         @serials = [{:type => 'pty', :source => nil}] if @serials == UNSET_VALUE
+
+        @host_device_exclude_prefixes = ['docker', 'macvtap', 'virbr', 'vnet'] if @host_device_exclude_prefixes == UNSET_VALUE
       end
 
       def validate(machine)
@@ -999,7 +1022,7 @@ module VagrantPlugins
         end
 
         if @qemu_use_agent == true
-          # if qemu agent is used to optain domain ip configuration, at least
+          # if qemu agent is used to obtain domain ip configuration, at least
           # one qemu channel has to be configured. As there are various options,
           # error out and leave configuration to the user
           unless machine.provider_config.channels.any? { |channel| channel[:target_name].start_with?("org.qemu.guest_agent") }
@@ -1029,7 +1052,9 @@ module VagrantPlugins
           errors << "#{e}"
         end
 
-        machine.config.vm.networks.each do |_type, opts|
+        machine.config.vm.networks.each_with_index do |network, index|
+          type, opts = network
+
           if opts[:mac]
             if opts[:mac] =~ /\A([0-9a-fA-F]{12})\z/
               opts[:mac] = opts[:mac].scan(/../).join(':')
@@ -1038,6 +1063,14 @@ module VagrantPlugins
               errors << "Configured NIC MAC '#{opts[:mac]}' is not in 'xx:xx:xx:xx:xx:xx' or 'xxxxxxxxxxxx' format"
             end
           end
+
+          # only interested in public networks where portgroup is nil, as then source will be a host device
+          if type == :public_network && opts[:portgroup] == nil
+            devices = host_devices(machine)
+            if !devices.include?(opts[:dev])
+              errors << "network configuration #{index} for machine #{machine.name} is a public_network referencing host device '#{opts[:dev]}' which does not exist, consider adding ':dev => ....' referencing one of #{devices.join(", ")}"
+            end
+          end
         end
 
         if !machine.provider_config.volume_cache.nil? and machine.provider_config.volume_cache != UNSET_VALUE
@@ -1048,11 +1081,15 @@ module VagrantPlugins
           end
         end
 
+        errors = validate_sysinfo(machine, errors)
+
         { 'Libvirt Provider' => errors }
       end
 
       def merge(other)
         super.tap do |result|
+          result.boot_order = other.boot_order != [] ? other.boot_order : boot_order
+
           c = disks.dup
           c += other.disks
           result.disks = c
@@ -1062,7 +1099,11 @@ module VagrantPlugins
           result.cdroms = c
 
           result.disk_driver_opts = disk_driver_opts.merge(other.disk_driver_opts)
-          
+
+          c = sysinfo == UNSET_VALUE ? {} : sysinfo.dup
+          c.merge!(other.sysinfo) { |_k, x, y| x.respond_to?(:each_pair) ? x.merge(y) : x + y } if other.sysinfo != UNSET_VALUE
+          result.sysinfo = c
+
           c = clock_timers.dup
           c += other.clock_timers
           result.clock_timers = c
@@ -1170,6 +1211,64 @@ module VagrantPlugins
           @proxy_command = nil
         end
       end
+
+      def host_devices(machine)
+        @host_devices ||= begin
+          (
+            machine.provider.driver.list_host_devices.map { |iface| iface.name } +
+            machine.provider.driver.list_networks.map { |net| net.bridge_name }
+          ).uniq.select do |dev|
+            next if dev.empty?
+            dev != "lo" && !@host_device_exclude_prefixes.any? { |exclude| dev.start_with?(exclude) }
+          end
+        end
+      end
+
+      def validate_sysinfo(machine, errors)
+        valid_sysinfo = {
+          'bios' => %w[vendor version date release],
+          'system' => %w[manufacturer product version serial uuid sku family],
+          'base board' => %w[manufacturer product version serial asset location],
+          'chassis' => %w[manufacturer version serial asset sku],
+          'oem strings' => nil,
+        }
+
+        machine.provider_config.sysinfo.each_pair do |block_name, entries|
+          block_name = block_name.to_s
+          unless valid_sysinfo.key?(block_name)
+            errors << "invalid sysinfo element '#{block_name}'; smbios sysinfo elements supported: #{valid_sysinfo.keys.join(', ')}"
+            next
+          end
+
+          if valid_sysinfo[block_name].nil?
+            # assume simple array of text entries
+            entries.each do |entry|
+              if entry.respond_to?(:to_str)
+                if entry.to_s.empty?
+                  machine.ui.warn("Libvirt Provider: 'sysinfo.#{block_name}' contains an empty or nil entry and will be discarded")
+                end
+              else
+                errors << "sysinfo.#{block_name} expects entries to be stringy, got #{entry.class} containing '#{entry}'"
+              end
+            end
+          else
+            entries.each_pair do |entry_name, entry_text|
+              entry_name = entry_name.to_s
+              unless valid_sysinfo[block_name].include?(entry_name)
+                errors << "'sysinfo.#{block_name}' does not support entry name '#{entry_name}'; entries supported: #{valid_sysinfo[block_name].join(', ')}"
+                next
+              end
+
+              # this allows removal of entries specified by other Vagrantfile's in the hierarchy
+              if entry_text.to_s.empty?
+                machine.ui.warn("Libvirt Provider: sysinfo.#{block_name}.#{entry_name} is nil or empty and therefore has no effect.")
+              end
+            end
+          end
+        end
+
+        errors
+      end
     end
   end
 end
